@@ -1,8 +1,7 @@
-/*
- * Copyright 2019 The Emscripten Authors.  All rights reserved.
- * Emscripten is available under two separate licenses, the MIT license and the
- * University of Illinois/NCSA Open Source License.  Both these licenses can be
- * found in the LICENSE file.
+/**
+ * @license
+ * Copyright 2019 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
  */
 
 var WasiLibrary = {
@@ -12,19 +11,24 @@ var WasiLibrary = {
     _exit(code);
   },
 
-  emscripten_get_environ__deps: ['$ENV', '_getExecutableName'],
-  emscripten_get_environ__sig: 'i',
-  emscripten_get_environ: function() {
-    if (!_emscripten_get_environ.strings) {
+  $getEnvStrings__deps: ['$ENV', '_getExecutableName'],
+  $getEnvStrings: function() {
+    if (!getEnvStrings.strings) {
       // Default values.
+#if !DETERMINISTIC
+      // Browser language detection #8751
+      var lang = ((typeof navigator === 'object' && navigator.languages && navigator.languages[0]) || 'C').replace('-', '_') + '.UTF-8';
+#else
+      // Deterministic language detection, ignore the browser's language.
+      var lang = 'C.UTF-8';
+#endif
       var env = {
         'USER': 'web_user',
         'LOGNAME': 'web_user',
         'PATH': '/',
         'PWD': '/',
         'HOME': '/home/web_user',
-        // Browser language detection #8751
-        'LANG': ((typeof navigator === 'object' && navigator.languages && navigator.languages[0]) || 'C').replace('-', '_') + '.UTF-8',
+        'LANG': lang,
         '_': __getExecutableName()
       };
       // Apply the user-provided values, if any.
@@ -35,15 +39,15 @@ var WasiLibrary = {
       for (var x in env) {
         strings.push(x + '=' + env[x]);
       }
-      _emscripten_get_environ.strings = strings;
+      getEnvStrings.strings = strings;
     }
-    return _emscripten_get_environ.strings;
+    return getEnvStrings.strings;
   },
 
-  environ_sizes_get__deps: ['emscripten_get_environ'],
+  environ_sizes_get__deps: ['$getEnvStrings'],
   environ_sizes_get__sig: 'iii',
   environ_sizes_get: function(penviron_count, penviron_buf_size) {
-    var strings = _emscripten_get_environ();
+    var strings = getEnvStrings();
     {{{ makeSetValue('penviron_count', 0, 'strings.length', 'i32') }}};
     var bufSize = 0;
     strings.forEach(function(string) {
@@ -53,16 +57,15 @@ var WasiLibrary = {
     return 0;
   },
 
-  environ_get__deps: ['emscripten_get_environ'
+  environ_get__deps: ['$getEnvStrings'
 #if MINIMAL_RUNTIME
     , '$writeAsciiToMemory'
 #endif
   ],
   environ_get__sig: 'iii',
   environ_get: function(__environ, environ_buf) {
-    var strings = _emscripten_get_environ();
     var bufSize = 0;
-    strings.forEach(function(string, i) {
+    getEnvStrings().forEach(function(string, i) {
       var ptr = environ_buf + bufSize;
       {{{ makeSetValue('__environ', 'i * 4', 'ptr', 'i32') }}};
       writeAsciiToMemory(string, ptr);
@@ -87,6 +90,9 @@ var WasiLibrary = {
   },
 
   args_get__sig: 'iii',
+#if MINIMAL_RUNTIME && MAIN_READS_PARAMS
+  args_get__deps: ['$writeAsciiToMemory'],
+#endif
   args_get: function(argv, argv_buf) {
 #if MAIN_READS_PARAMS
     var bufSize = 0;
@@ -100,21 +106,32 @@ var WasiLibrary = {
     return 0;
   },
 
+  $checkWasiClock: function(clock_id) {
+    return clock_id == {{{ cDefine('__WASI_CLOCKID_REALTIME') }}} ||
+           clock_id == {{{ cDefine('__WASI_CLOCKID_MONOTONIC') }}} ||
+           clock_id == {{{ cDefine('__WASI_CLOCKID_PROCESS_CPUTIME_ID') }}} ||
+           clock_id == {{{ cDefine('__WASI_CLOCKID_THREAD_CPUTIME_ID') }}};
+  },
+
   // TODO: the i64 in the API here must be legalized for this JS code to run,
   // but the wasm file can't be legalized in standalone mode, which is where
   // this is needed. To get this code to be usable as a JS shim we need to
   // either wait for BigInt support or to legalize on the client.
   clock_time_get__sig: 'iiiii',
-  clock_time_get__deps: ['emscripten_get_now', 'emscripten_get_now_is_monotonic', '__setErrNo'],
-  clock_time_get: function(clk_id, precision_l, precision_h, ptime) {
+  clock_time_get__deps: ['emscripten_get_now', 'emscripten_get_now_is_monotonic', '$setErrNo', '$checkWasiClock'],
+  clock_time_get: function(clk_id, {{{ defineI64Param('precision') }}}, ptime) {
+    {{{ receiveI64ParamAsI32s('precision') }}}
+    if (!checkWasiClock(clk_id)) {
+      return {{{ cDefine('EINVAL') }}};
+    }
     var now;
+    // all wasi clocks but realtime are monotonic
     if (clk_id === {{{ cDefine('__WASI_CLOCKID_REALTIME') }}}) {
       now = Date.now();
-    } else if (clk_id === {{{ cDefine('__WASI_CLOCKID_MONOTONIC') }}} && _emscripten_get_now_is_monotonic) {
+    } else if (_emscripten_get_now_is_monotonic) {
       now = _emscripten_get_now();
     } else {
-      ___setErrNo({{{ cDefine('EINVAL') }}});
-      return -1;
+      return {{{ cDefine('ENOSYS') }}};
     }
     // "now" is in ms, and wasi times are in ns.
     var nsec = Math.round(now * 1000 * 1000);
@@ -124,16 +141,19 @@ var WasiLibrary = {
   },
 
   clock_res_get__sig: 'iii',
-  clock_res_get__deps: ['emscripten_get_now', 'emscripten_get_now_is_monotonic', '__setErrNo'],
+  clock_res_get__deps: ['emscripten_get_now', 'emscripten_get_now_res', 'emscripten_get_now_is_monotonic', '$setErrNo', '$checkWasiClock'],
   clock_res_get: function(clk_id, pres) {
+    if (!checkWasiClock(clk_id)) {
+      return {{{ cDefine('EINVAL') }}};
+    }
     var nsec;
+    // all wasi clocks but realtime are monotonic
     if (clk_id === {{{ cDefine('CLOCK_REALTIME') }}}) {
       nsec = 1000 * 1000; // educated guess that it's milliseconds
-    } else if (clk_id === {{{ cDefine('CLOCK_MONOTONIC') }}} && _emscripten_get_now_is_monotonic) {
+    } else if (_emscripten_get_now_is_monotonic) {
       nsec = _emscripten_get_now_res();
     } else {
-      ___setErrNo({{{ cDefine('EINVAL') }}});
-      return -1;
+      return {{{ cDefine('ENOSYS') }}};
     }
     {{{ makeSetValue('pres', 0, 'nsec >>> 0', 'i32') }}};
     {{{ makeSetValue('pres', 4, '(nsec / Math.pow(2, 32)) >>> 0', 'i32') }}};
@@ -147,6 +167,7 @@ var WasiLibrary = {
 if (!WASM_BACKEND) {
   for (var x in WasiLibrary) {
     if (isJsLibraryConfigIdentifier(x)) continue;
+    if (isJsOnlyIdentifier(x)) continue;
     WasiLibrary['__wasi_' + x] = x;
   }
 }
