@@ -34,7 +34,6 @@ import multiprocessing
 import operator
 import os
 import random
-import re
 import shlex
 import shutil
 import string
@@ -58,16 +57,28 @@ else:
 __rootpath__ = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 sys.path.append(__rootpath__)
 
-import parallel_runner
-from tools.shared import EM_CONFIG, TEMP_DIR, EMCC, DEBUG, PYTHON, LLVM_TARGET, ASM_JS_TARGET, EMSCRIPTEN_TEMP_DIR, WASM_TARGET, SPIDERMONKEY_ENGINE, WINDOWS, EM_BUILD_VERBOSE
-from tools.shared import asstr, get_canonical_temp_dir, Building, run_process, try_delete, to_cc, asbytes, safe_copy, Settings
-from tools import jsrun, shared, line_endings
+import clang_native
+import jsrun
+import parallel_testsuite
+from jsrun import NON_ZERO
+from tools.shared import EM_CONFIG, TEMP_DIR, EMCC, EMXX, DEBUG
+from tools.shared import LLVM_TARGET, ASM_JS_TARGET, EMSCRIPTEN_TEMP_DIR
+from tools.shared import WASM_TARGET, SPIDERMONKEY_ENGINE, WINDOWS
+from tools.shared import EM_BUILD_VERBOSE
+from tools.shared import asstr, get_canonical_temp_dir, run_process, try_delete
+from tools.shared import asbytes, safe_copy, Settings
+from tools import shared, line_endings, building
 
 
 def path_from_root(*pathelems):
   return os.path.join(__rootpath__, *pathelems)
 
 
+def delete_contents(pathname):
+  for entry in os.listdir(pathname):
+    try_delete(os.path.join(pathname, entry))
+
+
 sys.path.append(path_from_root('third_party/websockify'))
 
 logger = logging.getLogger(__file__)
@@ -80,8 +91,10 @@ EMTEST_BROWSER = os.getenv('EMTEST_BROWSER')
 
 EMTEST_DETECT_TEMPFILE_LEAKS = int(os.getenv('EMTEST_DETECT_TEMPFILE_LEAKS', '0'))
 
-# Also suppot the old name: EM_SAVE_DIR
-EMTEST_SAVE_DIR = os.getenv('EMTEST_SAVE_DIR', os.getenv('EM_SAVE_DIR'))
+# TODO(sbc): Remove this check for the legacy name once its been around for a while.
+assert 'EM_SAVE_DIR' not in os.environ, "Please use EMTEST_SAVE_DIR instead of EM_SAVE_DIR"
+
+EMTEST_SAVE_DIR = int(os.getenv('EMTEST_SAVE_DIR', '0'))
 
 # generally js engines are equivalent, testing 1 is enough. set this
 # to force testing on all js engines, good to find js engine bugs
@@ -89,6 +102,8 @@ EMTEST_ALL_ENGINES = os.getenv('EMTEST_ALL_ENGINES')
 
 EMTEST_SKIP_SLOW = os.getenv('EMTEST_SKIP_SLOW')
 
+EMTEST_LACKS_NATIVE_CLANG = os.getenv('EMTEST_LACKS_NATIVE_CLANG')
+
 EMTEST_VERBOSE = int(os.getenv('EMTEST_VERBOSE', '0'))
 
 if EMTEST_VERBOSE:
@@ -172,6 +187,17 @@ def no_asmjs(note=''):
   return decorated
 
 
+def requires_native_clang(func):
+  assert callable(func)
+
+  def decorated(self, *args, **kwargs):
+    if EMTEST_LACKS_NATIVE_CLANG:
+      return self.skipTest('native clang tests are disabled')
+    return func(self, *args, **kwargs)
+
+  return decorated
+
+
 @contextlib.contextmanager
 def env_modify(updates):
   """A context manager that updates os.environ."""
@@ -214,6 +240,28 @@ def chdir(dir):
     os.chdir(orig_cwd)
 
 
+@contextlib.contextmanager
+def js_engines_modify(replacements):
+  """A context manager that updates shared.JS_ENGINES."""
+  original = shared.JS_ENGINES
+  shared.JS_ENGINES = replacements
+  try:
+    yield
+  finally:
+    shared.JS_ENGINES = original
+
+
+@contextlib.contextmanager
+def wasm_engines_modify(replacements):
+  """A context manager that updates shared.WASM_ENGINES."""
+  original = shared.WASM_ENGINES
+  shared.WASM_ENGINES = replacements
+  try:
+    yield
+  finally:
+    shared.WASM_ENGINES = original
+
+
 def ensure_dir(dirname):
   if not os.path.isdir(dirname):
     os.makedirs(dirname)
@@ -244,9 +292,10 @@ core_test_modes = [
   'wasm3',
   'wasms',
   'wasmz',
+  'strict'
 ]
 
-if shared.Settings.WASM_BACKEND:
+if Settings.WASM_BACKEND:
   core_test_modes += [
     'wasm2js0',
     'wasm2js1',
@@ -277,7 +326,7 @@ non_core_test_modes = [
   'benchmark',
 ]
 
-if shared.Settings.WASM_BACKEND:
+if Settings.WASM_BACKEND:
   non_core_test_modes += [
     'asan',
     'lsan',
@@ -331,7 +380,10 @@ class RunnerMeta(type):
       return func(self, *args)
 
     # Add suffix to the function name so that it displays correctly.
-    resulting_test.__name__ = '%s_%s' % (name, suffix)
+    if suffix:
+      resulting_test.__name__ = '%s_%s' % (name, suffix)
+    else:
+      resulting_test.__name__ = name
 
     # On python 3, functions have __qualname__ as well. This is a full dot-separated path to the function.
     # We add the suffix to it as well.
@@ -385,9 +437,6 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
   # Change this to None to get stderr reporting, for debugging purposes
   stderr_redirect = STDOUT
 
-  def is_emterpreter(self):
-    return self.get_setting('EMTERPRETIFY')
-
   def is_wasm(self):
     return self.get_setting('WASM') != 0
 
@@ -399,6 +448,8 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       self.skipTest('no dlfcn with memory growth (without wasm)')
     if self.get_setting('WASM_BACKEND') and not self.get_setting('WASM'):
       self.skipTest('no dynamic library support in wasm2js yet')
+    if '-fsanitize=address' in self.emcc_args:
+      self.skipTest('no dynamic library support in asan yet')
 
   def uses_memory_init_file(self):
     if self.get_setting('SIDE_MODULE') or \
@@ -427,10 +478,12 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     super(RunnerCore, self).setUp()
     self.settings_mods = {}
     self.emcc_args = ['-Werror']
-    self.save_dir = EMTEST_SAVE_DIR
     self.env = {}
     self.temp_files_before_run = []
 
+    if not Settings.WASM_BACKEND:
+      os.environ['EMCC_ALLOW_FASTCOMP'] = '1'
+
     if EMTEST_DETECT_TEMPFILE_LEAKS:
       for root, dirnames, filenames in os.walk(self.temp_dir):
         for dirname in dirnames:
@@ -440,21 +493,33 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
 
     self.banned_js_engines = []
     self.use_all_engines = EMTEST_ALL_ENGINES
-    if self.save_dir:
+    if EMTEST_SAVE_DIR:
       self.working_dir = os.path.join(self.temp_dir, 'emscripten_test')
-      ensure_dir(self.working_dir)
+      if os.path.exists(self.working_dir):
+        if EMTEST_SAVE_DIR == 2:
+          print('Not clearing existing test directory')
+        else:
+          print('Clearing existing test directory')
+          # Even when EMTEST_SAVE_DIR we still try to start with an empty directoy as many tests
+          # expect this.  EMTEST_SAVE_DIR=2 can be used to keep the old contents for the new test
+          # run. This can be useful when iterating on a given test with extra files you want to keep
+          # around in the output directory.
+          delete_contents(self.working_dir)
+      else:
+        print('Creating new test output directory')
+        ensure_dir(self.working_dir)
     else:
       self.working_dir = tempfile.mkdtemp(prefix='emscripten_test_' + self.__class__.__name__ + '_', dir=self.temp_dir)
     os.chdir(self.working_dir)
 
-    if not self.save_dir:
+    if not EMTEST_SAVE_DIR:
       self.has_prev_ll = False
       for temp_file in os.listdir(TEMP_DIR):
         if temp_file.endswith('.ll'):
           self.has_prev_ll = True
 
   def tearDown(self):
-    if not self.save_dir:
+    if not EMTEST_SAVE_DIR:
       # rmtree() fails on Windows if the current working directory is inside the tree.
       os.chdir(os.path.dirname(self.get_dir()))
       try_delete(self.get_dir())
@@ -467,9 +532,10 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
           for filename in filenames:
             temp_files_after_run.append(os.path.normpath(os.path.join(root, filename)))
 
-        # Our leak detection will pick up *any* new temp files in the temp dir. They may not be due to
-        # us, but e.g. the browser when running browser tests. Until we figure out a proper solution,
-        # ignore some temp file names that we see on our CI infrastructure.
+        # Our leak detection will pick up *any* new temp files in the temp dir.
+        # They may not be due to us, but e.g. the browser when running browser
+        # tests. Until we figure out a proper solution, ignore some temp file
+        # names that we see on our CI infrastructure.
         ignorable_file_prefixes = [
           '/tmp/tmpaddon',
           '/tmp/circleci-no-output-timeout',
@@ -484,12 +550,6 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
             print('leaked file: ' + f, file=sys.stderr)
           self.fail('Test leaked ' + str(len(left_over_files)) + ' temporary files!')
 
-      # Make sure we don't leave stuff around
-      # if not self.has_prev_ll:
-      #   for temp_file in os.listdir(TEMP_DIR):
-      #     assert not temp_file.endswith('.ll'), temp_file
-      #     # TODO assert not temp_file.startswith('emscripten_'), temp_file
-
   def get_setting(self, key):
     if key in self.settings_mods:
       return self.settings_mods[key]
@@ -521,8 +581,17 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
   def in_dir(self, *pathelems):
     return os.path.join(self.get_dir(), *pathelems)
 
-  def get_stdout_path(self):
-    return os.path.join(self.get_dir(), 'stdout')
+  def add_pre_run(self, code):
+    create_test_file('prerun.js', 'Module.preRun = function() { %s }' % code)
+    self.emcc_args += ['--pre-js', 'prerun.js']
+
+  def add_post_run(self, code):
+    create_test_file('postrun.js', 'Module.postRun = function() { %s }' % code)
+    self.emcc_args += ['--pre-js', 'postrun.js']
+
+  def add_on_exit(self, code):
+    create_test_file('onexit.js', 'Module.onExit = function() { %s }' % code)
+    self.emcc_args += ['--pre-js', 'onexit.js']
 
   def prep_ll_file(self, output_file, input_file, force_recompile=False, build_ll_hook=None):
     # force_recompile = force_recompile or os.path.getsize(filename + '.ll') > 50000
@@ -552,28 +621,28 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       if input_file.endswith(('.bc', '.o')):
         if input_file != output_obj:
           shutil.copy(input_file, output_obj)
-        Building.llvm_dis(output_obj, output_ll)
+        building.llvm_dis(output_obj, output_ll)
       else:
         shutil.copy(input_file, output_ll)
         fix_target(output_ll)
 
       if build_ll_hook:
         need_post = build_ll_hook(output_file)
-      Building.llvm_as(output_ll, output_obj)
+      building.llvm_as(output_ll, output_obj)
       shutil.move(output_ll, output_ll + '.pre') # for comparisons later
-      Building.llvm_dis(output_obj, output_ll)
+      building.llvm_dis(output_obj, output_ll)
       if build_ll_hook and need_post:
         build_ll_hook(output_file)
-        Building.llvm_as(output_ll, output_obj)
+        building.llvm_as(output_ll, output_obj)
         shutil.move(output_ll, output_ll + '.post') # for comparisons later
-        Building.llvm_dis(output_obj, output_ll)
+        building.llvm_dis(output_obj, output_ll)
 
-      Building.llvm_as(output_ll, output_obj)
+      building.llvm_as(output_ll, output_obj)
     else:
       if input_file.endswith('.ll'):
         safe_copy(input_file, output_ll)
         fix_target(output_ll)
-        Building.llvm_as(output_ll, output_obj)
+        building.llvm_as(output_ll, output_obj)
       else:
         safe_copy(input_file, output_obj)
 
@@ -620,18 +689,24 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       os.chdir(self.get_dir())
 
     suffix = '.o.js' if js_outfile else '.o.wasm'
+    all_sources = [filename] + additional_files
+    if any(os.path.splitext(s)[1] in ('.cc', '.cxx', '.cpp') for s in all_sources):
+      compiler = EMXX
+    else:
+      compiler = EMCC
+
     if build_ll_hook:
       # "slow", old path: build to bc, then build to JS
 
       # C++ => LLVM binary
 
-      for f in [filename] + additional_files:
+      for f in all_sources:
         try:
           # Make sure we notice if compilation steps failed
           os.remove(f + '.o')
         except OSError:
           pass
-        args = [PYTHON, EMCC] + self.get_emcc_args(main_file=True) + \
+        args = [compiler] + self.get_emcc_args(main_file=True) + \
                ['-I' + dirname, '-I' + os.path.join(dirname, 'include')] + \
                ['-I' + include for include in includes] + \
                ['-c', f, '-o', f + '.o']
@@ -643,7 +718,7 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       if len(additional_files) + len(libraries):
         shutil.move(object_file, object_file + '.alone')
         inputs = [object_file + '.alone'] + [f + '.o' for f in additional_files] + libraries
-        Building.link_to_object(inputs, object_file)
+        building.link_to_object(inputs, object_file)
         if not os.path.exists(object_file):
           print("Failed to link LLVM binaries:\n\n", object_file)
           self.fail("Linkage error")
@@ -652,15 +727,11 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       self.prep_ll_file(filename, object_file, build_ll_hook=build_ll_hook)
 
       # BC => JS
-      Building.emcc(object_file, self.get_emcc_args(main_file=True), object_file + '.js')
+      building.emcc(object_file, self.get_emcc_args(main_file=True), object_file + '.js')
     else:
       # "fast", new path: just call emcc and go straight to JS
-      all_files = [filename] + additional_files + libraries
-      for i in range(len(all_files)):
-        if '.' not in all_files[i]:
-          shutil.move(all_files[i], all_files[i] + '.bc')
-          all_files[i] += '.bc'
-      args = [PYTHON, EMCC] + self.get_emcc_args(main_file=True) + \
+      all_files = all_sources + libraries
+      args = [compiler] + self.get_emcc_args(main_file=True) + \
           ['-I' + dirname, '-I' + os.path.join(dirname, 'include')] + \
           ['-I' + include for include in includes] + \
           all_files + ['-o', filename + suffix]
@@ -677,23 +748,6 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       assert ('/* memory initializer */' not in src) or ('/* memory initializer */ allocate([]' in src)
 
   def validate_asmjs(self, err):
-    m = re.search(r"asm.js type error: '(\w+)' is not a (standard|supported) SIMD type", err)
-    if m:
-      # Bug numbers for missing SIMD types:
-      bugs = {
-        'Int8x16': 1136226,
-        'Int16x8': 1136226,
-        'Uint8x16': 1244117,
-        'Uint16x8': 1244117,
-        'Uint32x4': 1240796,
-        'Float64x2': 1124205,
-      }
-      simd = m.group(1)
-      if simd in bugs:
-        print(("\nWARNING: ignoring asm.js type error from {} due to implementation not yet available in SpiderMonkey." +
-               " See https://bugzilla.mozilla.org/show_bug.cgi?id={}\n").format(simd, bugs[simd]), file=sys.stderr)
-        err = err.replace(m.group(0), '')
-
     # check for asm.js validation
     if 'uccessfully compiled asm.js code' in err and 'asm.js link error' not in err:
       print("[was asm.js'ified]", file=sys.stderr)
@@ -734,7 +788,7 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     return num_funcs
 
   def count_wasm_contents(self, wasm_binary, what):
-    out = run_process([os.path.join(Building.get_binaryen_bin(), 'wasm-opt'), wasm_binary, '--metrics'], stdout=PIPE).stdout
+    out = run_process([os.path.join(building.get_binaryen_bin(), 'wasm-opt'), wasm_binary, '--metrics'], stdout=PIPE).stdout
     # output is something like
     # [?]        : 125
     for line in out.splitlines():
@@ -744,27 +798,27 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     self.fail('Failed to find [%s] in wasm-opt output' % what)
 
   def get_wasm_text(self, wasm_binary):
-    return run_process([os.path.join(Building.get_binaryen_bin(), 'wasm-dis'), wasm_binary], stdout=PIPE).stdout
+    return run_process([os.path.join(building.get_binaryen_bin(), 'wasm-dis'), wasm_binary], stdout=PIPE).stdout
 
   def is_exported_in_wasm(self, name, wasm):
     wat = self.get_wasm_text(wasm)
     return ('(export "%s"' % name) in wat
 
-  def run_generated_code(self, engine, filename, args=[], check_timeout=True, output_nicerizer=None, assert_returncode=0):
+  def run_js(self, filename, engine=None, args=[], output_nicerizer=None, assert_returncode=0):
     # use files, as PIPE can get too full and hang us
     stdout = self.in_dir('stdout')
     stderr = self.in_dir('stderr')
     # Make sure that we produced proper line endings to the .js file we are about to run.
-    self.assertEqual(line_endings.check_line_endings(filename), 0)
+    if not filename.endswith('.wasm'):
+      self.assertEqual(line_endings.check_line_endings(filename), 0)
     error = None
     if EMTEST_VERBOSE:
       print("Running '%s' under '%s'" % (filename, engine))
     try:
-      with chdir(self.get_dir()):
-        jsrun.run_js(filename, engine, args, check_timeout,
-                     stdout=open(stdout, 'w'),
-                     stderr=open(stderr, 'w'),
-                     assert_returncode=assert_returncode)
+      jsrun.run_js(filename, engine, args,
+                   stdout=open(stdout, 'w'),
+                   stderr=open(stderr, 'w'),
+                   assert_returncode=assert_returncode)
     except subprocess.CalledProcessError as e:
       error = e
 
@@ -781,7 +835,10 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       print(ret, end='')
       print('-- end program output --')
     if error:
-      self.fail('JS subprocess failed (%s): %s.  Output:\n%s' % (error.cmd, error.returncode, ret))
+      if assert_returncode == NON_ZERO:
+        self.fail('JS subprocess unexpectedly succeeded (%s):  Output:\n%s' % (error.cmd, ret))
+      else:
+        self.fail('JS subprocess failed (%s): %s.  Output:\n%s' % (error.cmd, error.returncode, ret))
 
     #  We should pass all strict mode checks
     self.assertNotContained('strict warning:', ret)
@@ -818,11 +875,11 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     for x in values:
       if x == y:
         return # success
-    diff_lines = difflib.unified_diff(x.split('\n'), y.split('\n'),
+    diff_lines = difflib.unified_diff(x.splitlines(), y.splitlines(),
                                       fromfile=fromfile, tofile=tofile)
     diff = ''.join([a.rstrip() + '\n' for a in diff_lines])
     if EMTEST_VERBOSE:
-      print("Expected to have '%s' == '%s'" % limit_size(values[0]), limit_size(y))
+      print("Expected to have '%s' == '%s'" % (limit_size(values[0]), limit_size(y)))
     fail_message = 'Unexpected difference:\n' + limit_size(diff)
     if not EMTEST_VERBOSE:
       fail_message += '\nFor full output run with EMTEST_VERBOSE=1.'
@@ -830,6 +887,10 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       fail_message += '\n' + msg
     self.fail(fail_message)
 
+  def assertIdenticalUrlEncoded(self, expected, actual, **kwargs):
+    """URL decodes the `actual` parameter before checking for equality."""
+    self.assertIdentical(expected, unquote(actual), **kwargs)
+
   def assertTextDataContained(self, text1, text2):
     text1 = text1.replace('\r\n', '\n')
     text2 = text2.replace('\r\n', '\n')
@@ -867,6 +928,12 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     else:
       self.assertNotContained(value, string)
 
+  def assertBinaryEqual(self, file1, file2):
+    self.assertEqual(os.path.getsize(file1),
+                     os.path.getsize(file2))
+    self.assertEqual(open(file1, 'rb').read(),
+                     open(file2, 'rb').read())
+
   library_cache = {}
 
   def get_build_dir(self):
@@ -878,7 +945,7 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
                   configure_args=[], make=['make'], make_args=None,
                   env_init={}, cache_name_extra='', native=False):
     if make_args is None:
-      make_args = ['-j', str(multiprocessing.cpu_count())]
+      make_args = ['-j', str(building.get_num_cores())]
 
     build_dir = self.get_build_dir()
     output_dir = self.get_dir()
@@ -999,7 +1066,10 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
 
         static const char *afunc_prev;
 
+        extern "C" {
         EMSCRIPTEN_KEEPALIVE void afunc(const char *s);
+        }
+
         void afunc(const char *s) {
           printf("a: %s (prev: %s)\n", s, afunc_prev);
           afunc_prev = s;
@@ -1017,8 +1087,11 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     create_test_file('libb.cpp', r'''
         #include <emscripten.h>
 
+        extern "C" {
         void afunc(const char *s);
         EMSCRIPTEN_KEEPALIVE void bfunc();
+        }
+
         void bfunc() {
           afunc("b");
         }
@@ -1027,8 +1100,11 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     create_test_file('libc.cpp', r'''
         #include <emscripten.h>
 
+        extern "C" {
         void afunc(const char *s);
         EMSCRIPTEN_KEEPALIVE void cfunc();
+        }
+
         void cfunc() {
           afunc("c");
         }
@@ -1046,7 +1122,7 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     so = '.wasm' if self.is_wasm() else '.js'
 
     def ccshared(src, linkto=[]):
-      cmdv = [PYTHON, EMCC, src, '-o', os.path.splitext(src)[0] + so] + self.get_emcc_args()
+      cmdv = [EMCC, src, '-o', os.path.splitext(src)[0] + so] + self.get_emcc_args()
       cmdv += ['-s', 'SIDE_MODULE=1', '-s', 'RUNTIME_LINKED_LIBS=' + str(linkto)]
       run_process(cmdv)
 
@@ -1057,10 +1133,12 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     self.set_setting('MAIN_MODULE', 1)
     self.set_setting('RUNTIME_LINKED_LIBS', ['libb' + so, 'libc' + so])
     do_run(r'''
+      extern "C" {
       void bfunc();
       void cfunc();
+      }
 
-      int _main() {
+      int test_main() {
         bfunc();
         cfunc();
         return 0;
@@ -1069,25 +1147,26 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
            'a: loaded\na: b (prev: (null))\na: c (prev: b)\n')
 
     self.set_setting('RUNTIME_LINKED_LIBS', [])
-    self.emcc_args += ['--embed-file', '.@/']
+    for libname in ['liba', 'libb', 'libc']:
+      self.emcc_args += ['--embed-file', libname + so]
     do_run(r'''
       #include <assert.h>
       #include <dlfcn.h>
       #include <stddef.h>
 
-      int _main() {
+      int test_main() {
         void *bdso, *cdso;
         void (*bfunc)(), (*cfunc)();
 
-        // FIXME for RTLD_LOCAL binding symbols to loaded lib is not currenlty working
+        // FIXME for RTLD_LOCAL binding symbols to loaded lib is not currently working
         bdso = dlopen("libb%(so)s", RTLD_GLOBAL);
         assert(bdso != NULL);
         cdso = dlopen("libc%(so)s", RTLD_GLOBAL);
         assert(cdso != NULL);
 
-        bfunc = (void (*)())dlsym(bdso, "_Z5bfuncv");
+        bfunc = (void (*)())dlsym(bdso, "bfunc");
         assert(bfunc != NULL);
-        cfunc = (void (*)())dlsym(cdso, "_Z5cfuncv");
+        cfunc = (void (*)())dlsym(cdso, "cfunc");
         assert(cfunc != NULL);
 
         bfunc();
@@ -1113,6 +1192,26 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     logger.debug('do_run_from_file: %s' % src)
     self.do_run(open(src).read(), open(expected_output).read(), *args, **kwargs)
 
+  def do_run_in_out_file_test(self, *path, **kwargs):
+    test_path = path_from_root(*path)
+
+    def find_files(*ext_list):
+      ret = None
+      count = 0
+      for ext in ext_list:
+        if os.path.isfile(test_path + ext):
+          ret = test_path + ext
+          count += 1
+      assert count > 0, ("No file found at {} with extension {}"
+                         .format(test_path, ext_list))
+      assert count <= 1, ("Test file {} found with multiple valid extensions {}"
+                          .format(test_path, ext_list))
+      return ret
+
+    src = find_files('.c', '.cpp')
+    output = find_files('.out', '.txt')
+    self.do_run_from_file(src, output, **kwargs)
+
   ## Does a complete test - builds, runs, checks output, etc.
   def do_run(self, src, expected_output, args=[], output_nicerizer=None,
              no_build=False, main_file=None, additional_files=[],
@@ -1122,7 +1221,6 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
              check_for_error=True):
     if force_c or (main_file is not None and main_file[-2:]) == '.c':
       basename = 'src.c'
-      Building.COMPILER = to_cc(Building.COMPILER)
 
     if no_build:
       if src:
@@ -1139,14 +1237,13 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       js_file = filename + '.o.js'
     self.assertExists(js_file)
 
-    # Run in both JavaScript engines, if optimizing - significant differences there (typed arrays)
-    js_engines = self.filtered_js_engines(js_engines)
+    engines = self.filtered_js_engines(js_engines)
     # Make sure to get asm.js validation checks, using sm, even if not testing all vms.
-    if len(js_engines) > 1 and not self.use_all_engines:
-      if SPIDERMONKEY_ENGINE in js_engines and not self.is_wasm_backend():
-        js_engines = [SPIDERMONKEY_ENGINE]
+    if len(engines) > 1 and not self.use_all_engines:
+      if SPIDERMONKEY_ENGINE in engines and not self.is_wasm_backend():
+        engines = [SPIDERMONKEY_ENGINE]
       else:
-        js_engines = js_engines[:1]
+        engines = engines[:1]
     # In standalone mode, also add wasm vms as we should be able to run there too.
     if self.get_setting('STANDALONE_WASM'):
       # TODO once standalone wasm support is more stable, apply use_all_engines
@@ -1154,11 +1251,20 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
       wasm_engines = shared.WASM_ENGINES
       if len(wasm_engines) == 0:
         logger.warning('no wasm engine was found to run the standalone part of this test')
-      js_engines += wasm_engines
-    if len(js_engines) == 0:
+      engines += wasm_engines
+      if self.get_setting('WASM2C') and not EMTEST_LACKS_NATIVE_CLANG:
+        # compile the c file to a native executable.
+        c = shared.unsuffixed(js_file) + '.wasm.c'
+        executable = shared.unsuffixed(js_file) + '.exe'
+        cmd = [shared.CLANG_CC, c, '-o', executable] + clang_native.get_clang_native_args()
+        shared.run_process(cmd, env=clang_native.get_clang_native_env())
+        # we can now run the executable directly, without an engine, which
+        # we indicate with None as the engine
+        engines += [[None]]
+    if len(engines) == 0:
       self.skipTest('No JS engine present to run this test with. Check %s and the paths therein.' % EM_CONFIG)
-    for engine in js_engines:
-      js_output = self.run_generated_code(engine, js_file, args, output_nicerizer=output_nicerizer, assert_returncode=assert_returncode)
+    for engine in engines:
+      js_output = self.run_js(js_file, engine, args, output_nicerizer=output_nicerizer, assert_returncode=assert_returncode)
       js_output = js_output.replace('\r\n', '\n')
       if expected_output:
         try:
@@ -1178,7 +1284,7 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
   def get_freetype_library(self):
     if '-Werror' in self.emcc_args:
       self.emcc_args.remove('-Werror')
-    return self.get_library('freetype', os.path.join('objs', '.libs', 'libfreetype.a'), configure_args=['--disable-shared', '--without-zlib'])
+    return self.get_library(os.path.join('third_party', 'freetype'), os.path.join('objs', '.libs', 'libfreetype.a'), configure_args=['--disable-shared', '--without-zlib'])
 
   def get_poppler_library(self, env_init=None):
     # The fontconfig symbols are all missing from the poppler build
@@ -1186,8 +1292,8 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     self.set_setting('ERROR_ON_UNDEFINED_SYMBOLS', 0)
 
     self.emcc_args += [
-      '-I' + path_from_root('tests', 'freetype', 'include'),
-      '-I' + path_from_root('tests', 'poppler', 'include')
+      '-I' + path_from_root('tests', 'third_party', 'freetype', 'include'),
+      '-I' + path_from_root('tests', 'third_party', 'poppler', 'include')
     ]
 
     freetype = self.get_freetype_library()
@@ -1208,7 +1314,7 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
     env_init['FONTCONFIG_LIBS'] = ' '
 
     poppler = self.get_library(
-        'poppler',
+        os.path.join('third_party', 'poppler'),
         [os.path.join('utils', 'pdftoppm.o'), os.path.join('utils', 'parseargs.o'), os.path.join('poppler', '.libs', 'libpoppler.a')],
         env_init=env_init,
         configure_args=['--disable-libjpeg', '--disable-libpng', '--disable-poppler-qt', '--disable-poppler-qt4', '--disable-cms', '--disable-cairo-output', '--disable-abiword-output', '--disable-shared'])
@@ -1217,12 +1323,12 @@ class RunnerCore(RunnerMeta('TestCase', (unittest.TestCase,), {})):
 
   def get_zlib_library(self):
     if WINDOWS:
-      return self.get_library('zlib', os.path.join('libz.a'),
+      return self.get_library(os.path.join('third_party', 'zlib'), os.path.join('libz.a'),
                               configure=[path_from_root('emconfigure.bat')],
                               configure_args=['cmake', '.'],
                               make=['mingw32-make'],
                               make_args=[])
-    return self.get_library('zlib', os.path.join('libz.a'), make_args=['libz.a'])
+    return self.get_library(os.path.join('third_party', 'zlib'), os.path.join('libz.a'), make_args=['libz.a'])
 
 
 # Run a server and a web page. When a test runs, we tell the server about it,
@@ -1266,6 +1372,10 @@ def harness_server_func(in_queue, out_queue, port):
         self.end_headers()
         self.wfile.write(open(path_from_root('tests', 'browser_harness.html'), 'rb').read())
       elif 'report_' in self.path:
+        # the test is reporting its result. first change dir away from the
+        # test dir, as it will be deleted now that the test is finishing, and
+        # if we got a ping at that time, we'd return an error
+        os.chdir(path_from_root())
         # for debugging, tests may encode the result and their own url (window.location) as result|url
         if '|' in self.path:
           path, url = self.path.split('|', 1)
@@ -1293,6 +1403,7 @@ def harness_server_func(in_queue, out_queue, port):
         self.send_header('Expires', '-1')
         self.end_headers()
         self.wfile.write(b'OK')
+
       elif 'stdout=' in self.path or 'stderr=' in self.path or 'exception=' in self.path:
         '''
           To get logging to the console from browser tests, add this to
@@ -1311,6 +1422,7 @@ def harness_server_func(in_queue, out_queue, port):
         self.send_header('Content-type', 'text/html')
         self.end_headers()
         if not in_queue.empty():
+          # there is a new test ready to be served
           url, dir = in_queue.get()
           if DEBUG:
             print('[queue command:', url, dir, ']')
@@ -1318,7 +1430,7 @@ def harness_server_func(in_queue, out_queue, port):
           assert out_queue.empty(), 'the single response from the last test was read'
           # tell the browser to load the test
           self.wfile.write(b'COMMAND:' + url)
-          # move us to the right place to serve the files
+          # move us to the right place to serve the files for the new test
           os.chdir(dir)
         else:
           # the browser must keep polling
@@ -1451,7 +1563,7 @@ class BrowserCore(RunnerCore):
         else:
           # verify the result, and try again if we should do so
           try:
-            self.assertIdentical(expectedResult, output)
+            self.assertIdenticalUrlEncoded(expectedResult, output)
           except Exception as e:
             if tries_left > 0:
               print('[test error (see below), automatically retrying]')
@@ -1591,7 +1703,7 @@ class BrowserCore(RunnerCore):
 ''' % (reporting.read(), basename, int(manually_trigger)))
 
   def compile_btest(self, args):
-    run_process([PYTHON, EMCC] + args + ['--pre-js', path_from_root('tests', 'browser_reporting.js')])
+    run_process([EMCC] + args + ['--pre-js', path_from_root('tests', 'browser_reporting.js')])
 
   def btest(self, filename, expected=None, reference=None, force_c=False,
             reference_slack=0, manual_reference=False, post_build=None,
@@ -1603,9 +1715,6 @@ class BrowserCore(RunnerCore):
     filename_is_src = '\n' in filename
     src = filename if filename_is_src else ''
     original_args = args[:]
-    if 'USE_PTHREADS=1' in args and self.is_wasm_backend():
-        # wasm2js does not support threads yet
-        also_asmjs = False
     if 'WASM=0' not in args:
       # Filter out separate-asm, which is implied by wasm
       args = [a for a in args if a != '--separate-asm']
@@ -1641,6 +1750,7 @@ class BrowserCore(RunnerCore):
 
     # Tests can opt into being run under asmjs as well
     if 'WASM=0' not in args and (also_asmjs or self.also_asmjs):
+      print('WASM=0')
       self.btest(filename, expected, reference, force_c, reference_slack, manual_reference, post_build,
                  original_args + ['-s', 'WASM=0'], outfile, message, also_proxied=False, timeout=timeout)
 
@@ -1672,11 +1782,11 @@ def build_library(name,
                   env_init={},
                   native=False,
                   cflags=[]):
-  """Build a library into a .bc file. We build the .bc file once and cache it
-  for all our tests. (We cache in memory since the test directory is destroyed
-  and recreated for each test. Note that we cache separately for different
-  compilers).  This cache is just during the test runner. There is a different
-  concept of caching as well, see |Cache|.
+  """Build a library and cache the result.  We build the library file
+  once and cache it for all our tests. (We cache in memory since the test
+  directory is destroyed and recreated for each test. Note that we cache
+  separately for different compilers).  This cache is just during the test
+  runner. There is a different concept of caching as well, see |Cache|.
   """
 
   if type(generated_libs) is not list:
@@ -1690,7 +1800,10 @@ def build_library(name,
   shutil.copytree(source_dir, project_dir) # Useful in debugging sometimes to comment this out, and two lines above
 
   generated_libs = [os.path.join(project_dir, lib) for lib in generated_libs]
-  env = Building.get_building_env(native, True, cflags=cflags)
+  if native:
+    env = clang_native.get_clang_native_env()
+  else:
+    env = building.get_building_env(cflags=cflags)
   for k, v in env_init.items():
     env[k] = v
   if configure:
@@ -1699,7 +1812,7 @@ def build_library(name,
         with open(os.path.join(project_dir, 'configure_err'), 'w') as err:
           stdout = out if EM_BUILD_VERBOSE < 2 else None
           stderr = err if EM_BUILD_VERBOSE < 1 else None
-          Building.configure(configure + configure_args, env=env,
+          building.configure(configure + configure_args, env=env,
                              stdout=stdout,
                              stderr=stderr,
                              cwd=project_dir)
@@ -1728,7 +1841,7 @@ def build_library(name,
       with open_make_err('w') as make_err:
         stdout = make_out if EM_BUILD_VERBOSE < 2 else None
         stderr = make_err if EM_BUILD_VERBOSE < 1 else None
-        Building.make(make + make_args, stdout=stdout, stderr=stderr, env=env,
+        building.make(make + make_args, stdout=stdout, stderr=stderr, env=env,
                       cwd=project_dir)
   except subprocess.CalledProcessError:
     with open_make_out() as f:
@@ -1927,10 +2040,11 @@ def flattened_tests(loaded_tests):
 
 def suite_for_module(module, tests):
   suite_supported = module.__name__ in ('test_core', 'test_other')
-  has_multiple_tests = len(tests) > 1
-  has_multiple_cores = parallel_runner.num_cores() > 1
-  if suite_supported and has_multiple_tests and has_multiple_cores:
-    return parallel_runner.ParallelTestSuite()
+  if not EMTEST_SAVE_DIR:
+    has_multiple_tests = len(tests) > 1
+    has_multiple_cores = parallel_testsuite.num_cores() > 1
+    if suite_supported and has_multiple_tests and has_multiple_cores:
+      return parallel_testsuite.ParallelTestSuite(len(tests))
   return unittest.TestSuite()
 
 
